@@ -30,30 +30,17 @@
 #include "registry.h"
 #include "introspection.h"
 
-typedef struct event_data event_data;
-struct event_data
+typedef struct
 {
   gchar *listener_bus_name;
   gchar *app_bus_name;
   gchar **data;
   GSList *properties;
-};
-
-static void
-children_added_listener (DBusConnection * bus,
-                         gint             index,
-                         const gchar    * name,
-                         const gchar    * path);
-
-static void
-children_removed_listener (DBusConnection * bus,
-                           gint             index,
-                           const gchar    * name,
-                           const gchar    * path);
+} EventData;
 
 /*---------------------------------------------------------------------------*/
 
-typedef struct _SpiReference
+typedef struct
 {
   gchar *name;
   gchar *path;
@@ -83,10 +70,25 @@ spi_reference_free (SpiReference *ref)
 
 G_DEFINE_TYPE(SpiRegistry, spi_registry, G_TYPE_OBJECT)
 
+static void
+spi_registry_finalize (GObject *object)
+{
+  SpiRegistry *registry = SPI_REGISTRY (object);
+
+  g_clear_pointer (&registry->bus_unique_name, g_free);
+
+  G_OBJECT_CLASS (spi_registry_parent_class)->finalize (object);
+}
+
 static void
 spi_registry_class_init (SpiRegistryClass *klass)
 {
+  GObjectClass *gobject_class;
+
   spi_registry_parent_class = g_type_class_ref (G_TYPE_OBJECT);
+
+  gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->finalize = spi_registry_finalize;
 }
 
 static void
@@ -162,21 +164,58 @@ find_index_of_reference (GPtrArray *arr, const gchar *name, const gchar * path,
 }
 
 static void
-add_application (SpiRegistry *reg, DBusConnection *bus, const gchar *name, const gchar *path)
+emit_event (DBusConnection *bus,
+            const char *iface_name,
+            const char *signal_name,
+            const char *detail_str,
+            dbus_int32_t detail1,
+            dbus_int32_t detail2,
+            SpiReference *app)
 {
-  g_ptr_array_add (reg->apps, spi_reference_new (name, path));
-  children_added_listener (bus, reg->apps->len - 1, name, path);
+  DBusMessage *sig;
+  DBusMessageIter iter, iter_variant, iter_array;
+
+  sig = dbus_message_new_signal(SPI_DBUS_PATH_ROOT, iface_name, signal_name);
+
+  dbus_message_iter_init_append(sig, &iter);
+
+  dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &detail_str);
+  dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &detail1);
+  dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &detail2);
+
+  dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, "(so)",
+                                    &iter_variant);
+    append_reference (&iter_variant, app->name, app->path);
+  dbus_message_iter_close_container (&iter, &iter_variant);
+
+  dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{sv}",
+                                    &iter_array);
+  dbus_message_iter_close_container (&iter, &iter_array);
+
+  dbus_connection_send(bus, sig, NULL);
+  dbus_message_unref(sig);
 }
 
 static void
-set_id (SpiRegistry *reg, DBusConnection *bus, const gchar *name, const gchar *path)
+add_application (SpiRegistry *registry, SpiReference *app_root)
+{
+  gint index;
+
+  g_ptr_array_add (registry->apps, app_root);
+  index = registry->apps->len - 1;
+
+  emit_event (registry->bus, SPI_DBUS_INTERFACE_EVENT_OBJECT, "ChildrenChanged", "add", index, 0, app_root);
+}
+
+static void
+set_id (SpiRegistry *registry, SpiReference *app)
 {
   DBusMessage *message;
   DBusMessageIter iter, iter_variant;
   const char *iface_application = "org.a11y.atspi.Application";
   const char *id = "Id";
 
-  message = dbus_message_new_method_call (name, path,
+  message = dbus_message_new_method_call (app->name, app->path,
                                           DBUS_INTERFACE_PROPERTIES, "Set");
   if (!message)
     return;
@@ -184,22 +223,22 @@ set_id (SpiRegistry *reg, DBusConnection *bus, const gchar *name, const gchar *p
   dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &iface_application);
   dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &id);
   dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, "i", &iter_variant);
-  dbus_message_iter_append_basic (&iter_variant, DBUS_TYPE_INT32, &reg->id);
+  dbus_message_iter_append_basic (&iter_variant, DBUS_TYPE_INT32, &registry->id);
   /* TODO: This will cause problems if we cycle through 2^31 ids */
-  reg->id++;
+  registry->id++;
   dbus_message_iter_close_container (&iter, &iter_variant);
-  dbus_connection_send (bus, message, NULL);
+  dbus_connection_send (registry->bus, message, NULL);
   dbus_message_unref (message);
 }
 
 static void
-remove_application (SpiRegistry *reg, DBusConnection *bus, guint index)
+remove_application (SpiRegistry *registry, guint index)
 {
-  SpiReference *ref = g_ptr_array_index (reg->apps, index);
+  SpiReference *ref = g_ptr_array_index (registry->apps, index);
 
-  spi_remove_device_listeners (reg->dec, ref->name);
-  children_removed_listener (bus, index, ref->name, ref->path);
-  g_ptr_array_remove_index (reg->apps, index);
+  spi_remove_device_listeners (registry->dec, ref->name);
+  emit_event (registry->bus, SPI_DBUS_INTERFACE_EVENT_OBJECT, "ChildrenChanged", "remove", index, 0, ref);
+  g_ptr_array_remove_index (registry->apps, index);
 }
 
 static gboolean
@@ -241,7 +280,7 @@ remove_events (SpiRegistry *registry, const char *bus_name, const char *event)
 
   for (list = registry->events; list;)
     {
-      event_data *evdata = list->data;
+      EventData *evdata = list->data;
       list = list->next;
       if (!g_strcmp0 (evdata->listener_bus_name, bus_name) &&
           event_is_subtype (evdata->data, remove_data))
@@ -275,10 +314,9 @@ remove_events (SpiRegistry *registry, const char *bus_name, const char *event)
 }
 
 static void
-handle_disconnection (DBusConnection *bus, DBusMessage *message, void *user_data)
+handle_disconnection (SpiRegistry *registry, DBusMessage *message)
 {
   char *name, *old, *new;
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
 
   if (dbus_message_get_args (message, NULL,
                              DBUS_TYPE_STRING, &name,
@@ -290,17 +328,17 @@ handle_disconnection (DBusConnection *bus, DBusMessage *message, void *user_data
         {
           /* Remove all children with the application name the same as the disconnected application. */
           guint i;
-          for (i = 0; i < reg->apps->len; i++)
+          for (i = 0; i < registry->apps->len; i++)
             {
-              SpiReference *ref  = g_ptr_array_index (reg->apps, i);
+              SpiReference *ref = g_ptr_array_index (registry->apps, i);
               if (!g_strcmp0 (old, ref->name))
                 {
-                  remove_application (reg, bus, i);
+                  remove_application (registry, i);
                   i--;
                 }
-            } 
+            }
 
-          remove_events (reg, old, "");
+          remove_events (registry, old, "");
         }
     }
 }
@@ -343,7 +381,9 @@ ensure_proper_format (const char *name)
 static DBusHandlerResult
 signal_filter (DBusConnection *bus, DBusMessage *message, void *user_data)
 {
+  SpiRegistry *reg = SPI_REGISTRY (user_data);
   guint res = DBUS_HANDLER_RESULT_HANDLED;
+
   const gint   type    = dbus_message_get_type (message);
   const char *iface = dbus_message_get_interface (message);
   const char *member = dbus_message_get_member (message);
@@ -353,58 +393,83 @@ signal_filter (DBusConnection *bus, DBusMessage *message, void *user_data)
 
   if (!g_strcmp0(iface, DBUS_INTERFACE_DBUS) &&
       !g_strcmp0(member, "NameOwnerChanged"))
-      handle_disconnection (bus, message, user_data);
+      handle_disconnection (reg, message);
   else
       res = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
   return res;
 }
 
+typedef enum {
+  DEMARSHAL_STATUS_SUCCESS,
+  DEMARSHAL_STATUS_INVALID_SIGNATURE,
+  DEMARSHAL_STATUS_INVALID_VALUE,
+} DemarshalStatus;
+
 /* org.at_spi.Socket interface */
 /*---------------------------------------------------------------------------*/
 
-static DBusMessage*
-impl_Embed (DBusConnection *bus, DBusMessage *message, void *user_data)
+static DemarshalStatus
+socket_embed_demarshal (DBusMessage *message, SpiReference **out_app_root)
 {
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
   DBusMessageIter iter, iter_struct;
   const gchar *app_name, *obj_path;
-
-  DBusMessage *reply = NULL;
-  DBusMessageIter reply_iter;
+  SpiReference *app_root;
 
   dbus_message_iter_init (message, &iter);
   dbus_message_iter_recurse (&iter, &iter_struct);
   if (!(dbus_message_iter_get_arg_type (&iter_struct) == DBUS_TYPE_STRING))
-	goto error;
+    return DEMARSHAL_STATUS_INVALID_SIGNATURE;
   dbus_message_iter_get_basic (&iter_struct, &app_name);
   if (!app_name)
     app_name = dbus_message_get_sender (message);
   if (!dbus_message_iter_next (&iter_struct))
-        goto error;
+    return DEMARSHAL_STATUS_INVALID_SIGNATURE;
   if (!(dbus_message_iter_get_arg_type (&iter_struct) == DBUS_TYPE_OBJECT_PATH))
-	goto error;
+    return DEMARSHAL_STATUS_INVALID_SIGNATURE;
   dbus_message_iter_get_basic (&iter_struct, &obj_path);
 
-  add_application(reg, bus, app_name, obj_path);
+  app_root = spi_reference_new (app_name, obj_path);
+  *out_app_root = app_root;
 
-  set_id (reg, bus, app_name, obj_path);
+  return DEMARSHAL_STATUS_SUCCESS;
+}
 
-  reply = dbus_message_new_method_return (message);
-  dbus_message_iter_init_append (reply, &reply_iter);
-  append_reference (&reply_iter, 
-                    dbus_bus_get_unique_name (bus),
-                    SPI_DBUS_PATH_ROOT);
-
-  return reply;
-error:
-  return dbus_message_new_error (message, DBUS_ERROR_FAILED, "Invalid arguments");
+static SpiReference *
+socket_embed (SpiRegistry *registry, SpiReference *app_root)
+{
+  add_application (registry, app_root);
+  set_id (registry, app_root);
+  return spi_reference_new (registry->bus_unique_name, SPI_DBUS_PATH_ROOT);
 }
 
 static DBusMessage*
-impl_Unembed (DBusConnection *bus, DBusMessage *message, void *user_data)
+impl_Embed (DBusMessage *message, SpiRegistry *registry)
+{
+  SpiReference *app_root = NULL;
+  SpiReference *result;
+
+  if (socket_embed_demarshal (message, &app_root) != DEMARSHAL_STATUS_SUCCESS)
+    {
+      return dbus_message_new_error (message, DBUS_ERROR_FAILED, "Invalid arguments");
+    }
+
+  DBusMessage *reply = NULL;
+  DBusMessageIter reply_iter;
+
+  result = socket_embed (registry, app_root); /* takes ownership of the app_root */
+
+  reply = dbus_message_new_method_return (message);
+  dbus_message_iter_init_append (reply, &reply_iter);
+  append_reference (&reply_iter, result->name, result->path);
+  spi_reference_free (result);
+
+  return reply;
+}
+
+static DBusMessage*
+impl_Unembed (DBusMessage *message, SpiRegistry *registry)
 {
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
   DBusMessageIter iter, iter_struct;
   gchar *app_name, *obj_path;
   guint index;
@@ -420,8 +485,8 @@ impl_Unembed (DBusConnection *bus, DBusMessage *message, void *user_data)
 	goto error;
   dbus_message_iter_get_basic (&iter_struct, &obj_path);
 
-  if (find_index_of_reference (reg->apps, app_name, obj_path, &index))
-      remove_application(reg, bus, index);
+  if (find_index_of_reference (registry->apps, app_name, obj_path, &index))
+    remove_application (registry, index);
 
   return NULL;
 error:
@@ -432,7 +497,7 @@ error:
 /*---------------------------------------------------------------------------*/
 
 static DBusMessage *
-impl_Contains (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_Contains (DBusMessage * message, SpiRegistry *registry)
 {
   dbus_bool_t retval = FALSE;
   DBusMessage *reply;
@@ -444,8 +509,7 @@ impl_Contains (DBusConnection * bus, DBusMessage * message, void *user_data)
 }
 
 static DBusMessage *
-impl_GetAccessibleAtPoint (DBusConnection * bus, DBusMessage * message,
-                           void *user_data)
+impl_GetAccessibleAtPoint (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply = NULL;
   DBusMessageIter iter;
@@ -453,14 +517,14 @@ impl_GetAccessibleAtPoint (DBusConnection * bus, DBusMessage * message,
   reply = dbus_message_new_method_return (message);
   dbus_message_iter_init_append (reply, &iter);
   append_reference (&iter, 
-                    dbus_bus_get_unique_name (bus),
+                    registry->bus_unique_name,
                     SPI_DBUS_PATH_NULL);
 
   return reply;
 }
 
 static DBusMessage *
-impl_GetExtents (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetExtents (DBusMessage * message, SpiRegistry *registry)
 {
   dbus_int32_t x = 0, y = 0, width = 1024, height = 768;
   DBusMessage *reply;
@@ -479,8 +543,7 @@ impl_GetExtents (DBusConnection * bus, DBusMessage * message, void *user_data)
 }
 
 static DBusMessage *
-impl_GetPosition (DBusConnection * bus, DBusMessage * message,
-                  void *user_data)
+impl_GetPosition (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_int32_t x = 0, y = 0;
@@ -492,7 +555,7 @@ impl_GetPosition (DBusConnection * bus, DBusMessage * message,
 }
 
 static DBusMessage *
-impl_GetSize (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetSize (DBusMessage * message, SpiRegistry *registry)
 {
   /* TODO - Get the screen size */
   DBusMessage *reply;
@@ -507,7 +570,7 @@ impl_GetSize (DBusConnection * bus, DBusMessage * message, void *user_data)
 #define LAYER_WIDGET 3;
 
 static DBusMessage *
-impl_GetLayer (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetLayer (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_uint32_t rv = LAYER_WIDGET;
@@ -519,8 +582,7 @@ impl_GetLayer (DBusConnection * bus, DBusMessage * message, void *user_data)
 }
 
 static DBusMessage *
-impl_GetMDIZOrder (DBusConnection * bus, DBusMessage * message,
-                   void *user_data)
+impl_GetMDIZOrder (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_int16_t rv = 0;
@@ -532,7 +594,7 @@ impl_GetMDIZOrder (DBusConnection * bus, DBusMessage * message,
 }
 
 static DBusMessage *
-impl_GrabFocus (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GrabFocus (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_bool_t retval = FALSE;
@@ -544,7 +606,7 @@ impl_GrabFocus (DBusConnection * bus, DBusMessage * message, void *user_data)
 }
 
 static DBusMessage *
-impl_GetAlpha (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetAlpha (DBusMessage * message, SpiRegistry *registry)
 {
   double rv = 1.0;
   DBusMessage *reply;
@@ -559,21 +621,21 @@ impl_GetAlpha (DBusConnection * bus, DBusMessage * message, void *user_data)
 /*---------------------------------------------------------------------------*/
 
 static dbus_bool_t
-impl_get_Name (DBusMessageIter * iter, void *user_data)
+impl_get_Name (DBusMessageIter * iter, SpiRegistry *registry)
 {
   const gchar *name = "main";
   return return_v_string (iter, name);
 }
 
 static dbus_bool_t
-impl_get_Description (DBusMessageIter * iter, void *user_data)
+impl_get_Description (DBusMessageIter * iter, SpiRegistry *registry)
 {
   const gchar *description = "";
   return return_v_string (iter, description);
 }
 
 static dbus_bool_t
-impl_get_Parent (DBusMessageIter * iter, void *user_data)
+impl_get_Parent (DBusMessageIter * iter, SpiRegistry *registry)
 {
   const gchar *name = "";
   DBusMessageIter iter_variant;
@@ -588,10 +650,9 @@ impl_get_Parent (DBusMessageIter * iter, void *user_data)
 }
 
 static dbus_bool_t
-impl_get_ChildCount (DBusMessageIter * iter, void *user_data)
+impl_get_ChildCount (DBusMessageIter * iter, SpiRegistry *registry)
 {
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
-  dbus_int32_t rv = reg->apps->len;
+  dbus_int32_t rv = registry->apps->len;
   dbus_bool_t result;
   DBusMessageIter iter_variant;
 
@@ -604,22 +665,20 @@ impl_get_ChildCount (DBusMessageIter * iter, void *user_data)
 }
 
 static dbus_bool_t
-impl_get_ToolkitName (DBusMessageIter * iter, void *user_data)
+impl_get_ToolkitName (DBusMessageIter * iter, SpiRegistry *registry)
 {
   return return_v_string (iter, "at-spi-registry");
 }
 
 static dbus_bool_t
-impl_get_ToolkitVersion (DBusMessageIter * iter, void *user_data)
+impl_get_ToolkitVersion (DBusMessageIter * iter, SpiRegistry *registry)
 {
   return return_v_string (iter, "2.0");
 }
 
 static DBusMessage *
-impl_GetChildAtIndex (DBusConnection * bus,
-                      DBusMessage * message, void *user_data)
+impl_GetChildAtIndex (DBusMessage * message, SpiRegistry *registry)
 {
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
   DBusMessage *reply;
   DBusMessageIter iter;
   DBusError error;
@@ -636,11 +695,11 @@ impl_GetChildAtIndex (DBusConnection * bus,
   reply = dbus_message_new_method_return (message);
   dbus_message_iter_init_append (reply, &iter);
 
-  if (i < 0 || i >= reg->apps->len)
+  if (i < 0 || i >= registry->apps->len)
     append_reference (&iter, SPI_DBUS_NAME_REGISTRY, SPI_DBUS_PATH_NULL);
   else
     {
-      ref = g_ptr_array_index (reg->apps, i);
+      ref = g_ptr_array_index (registry->apps, i);
       append_reference (&iter, ref->name, ref->path);
     }
 
@@ -648,21 +707,19 @@ impl_GetChildAtIndex (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetChildren (DBusConnection * bus,
-                  DBusMessage * message, void *user_data)
+impl_GetChildren (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply = NULL;
   DBusMessageIter iter, iter_array;
-  SpiRegistry *reg = SPI_REGISTRY (user_data);
   int i;
 
   reply = dbus_message_new_method_return (message);
 
   dbus_message_iter_init_append (reply, &iter);
   dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "(so)", &iter_array);
-  for (i=0; i < reg->apps->len; i++)
+  for (i=0; i < registry->apps->len; i++)
     {
-      SpiReference *current = g_ptr_array_index (reg->apps, i);
+      SpiReference *current = g_ptr_array_index (registry->apps, i);
       append_reference (&iter_array, current->name, current->path);
     }
   dbus_message_iter_close_container(&iter, &iter_array);
@@ -670,8 +727,7 @@ impl_GetChildren (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetIndexInParent (DBusConnection * bus,
-                       DBusMessage * message, void *user_data)
+impl_GetIndexInParent (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_uint32_t rv = 0;
@@ -682,8 +738,7 @@ impl_GetIndexInParent (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetRelationSet (DBusConnection * bus,
-                     DBusMessage * message, void *user_data)
+impl_GetRelationSet (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   DBusMessageIter iter, iter_array;
@@ -697,7 +752,7 @@ impl_GetRelationSet (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetRole (DBusConnection * bus, DBusMessage * message, void * user_data)
+impl_GetRole (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   dbus_uint32_t rv = 14;	/* TODO: Get DESKTOP_FRAME from somewhere */
@@ -708,8 +763,7 @@ impl_GetRole (DBusConnection * bus, DBusMessage * message, void * user_data)
 }
 
 static DBusMessage *
-impl_GetRoleName (DBusConnection * bus,
-                  DBusMessage * message, void *user_data)
+impl_GetRoleName (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   const char *role_name = "desktop frame";
@@ -721,8 +775,7 @@ impl_GetRoleName (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetLocalizedRoleName (DBusConnection * bus,
-                           DBusMessage * message, void *user_data)
+impl_GetLocalizedRoleName (DBusMessage * message, SpiRegistry *registry)
 {
   /* TODO - Localize this */
   DBusMessage *reply;
@@ -735,7 +788,7 @@ impl_GetLocalizedRoleName (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetState (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetState (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply = NULL;
   DBusMessageIter iter, iter_array;
@@ -757,8 +810,7 @@ impl_GetState (DBusConnection * bus, DBusMessage * message, void *user_data)
 }
 
 static DBusMessage *
-impl_GetAttributes (DBusConnection * bus,
-                    DBusMessage * message, void *user_data)
+impl_GetAttributes (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply = NULL;
   DBusMessageIter iter, array;
@@ -772,8 +824,7 @@ impl_GetAttributes (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetApplication (DBusConnection * bus,
-                     DBusMessage * message, void *user_data)
+impl_GetApplication (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply = NULL;
   DBusMessageIter iter;
@@ -781,15 +832,14 @@ impl_GetApplication (DBusConnection * bus,
   reply = dbus_message_new_method_return (message);
   dbus_message_iter_init_append (reply, &iter);
   append_reference (&iter,
-                    dbus_bus_get_unique_name (bus),
+                    registry->bus_unique_name,
                     SPI_DBUS_PATH_NULL);
 
   return reply;
 }
 
 static DBusMessage *
-impl_GetInterfaces (DBusConnection * bus,
-                    DBusMessage * message, void *user_data)
+impl_GetInterfaces (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   DBusMessageIter iter, iter_array;
@@ -810,7 +860,7 @@ impl_GetInterfaces (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_GetItems (DBusConnection * bus, DBusMessage * message, void *user_data)
+impl_GetItems (DBusMessage * message, SpiRegistry *registry)
 {
   DBusMessage *reply;
   DBusMessageIter iter, iter_array;
@@ -829,12 +879,11 @@ impl_GetItems (DBusConnection * bus, DBusMessage * message, void *user_data)
  * a method call and signal for now.
  */
 static DBusMessage *
-impl_register_event (DBusConnection *bus, DBusMessage *message, void *user_data)
+impl_RegisterEvent (DBusMessage *message, SpiRegistry *registry)
 {
-  SpiRegistry *registry = SPI_REGISTRY (user_data);
   const char *orig_name;
   gchar *name;
-  event_data *evdata;
+  EventData *evdata;
   gchar **data;
   DBusMessage *signal;
   const char *sender = dbus_message_get_sender (message);
@@ -854,7 +903,7 @@ impl_register_event (DBusConnection *bus, DBusMessage *message, void *user_data)
   dbus_message_iter_next (&iter);
   name = ensure_proper_format (orig_name);
 
-  evdata = g_new0 (event_data, 1);
+  evdata = g_new0 (EventData, 1);
   data = g_strsplit (name, ":", 3);
   evdata->listener_bus_name = g_strdup (sender);
   evdata->data = data;
@@ -907,7 +956,7 @@ impl_register_event (DBusConnection *bus, DBusMessage *message, void *user_data)
       ls = g_slist_next (ls);
     }
     dbus_message_iter_close_container (&iter, &iter_array);
-    dbus_connection_send (bus, signal, NULL);
+    dbus_connection_send (registry->bus, signal, NULL);
     dbus_message_unref (signal);
   }
 
@@ -916,9 +965,8 @@ impl_register_event (DBusConnection *bus, DBusMessage *message, void *user_data)
 }
 
 static DBusMessage *
-impl_deregister_event (DBusConnection *bus, DBusMessage *message, void *user_data)
+impl_DeregisterEvent (DBusMessage *message, SpiRegistry *registry)
 {
-  SpiRegistry *registry = SPI_REGISTRY (user_data);
   const char *orig_name;
   gchar *name;
   const char *sender = dbus_message_get_sender (message);
@@ -935,10 +983,9 @@ impl_deregister_event (DBusConnection *bus, DBusMessage *message, void *user_dat
 }
 
 static DBusMessage *
-impl_get_registered_events (DBusConnection *bus, DBusMessage *message, void *user_data)
+impl_GetRegisteredEvents (DBusMessage *message, SpiRegistry *registry)
 {
-  SpiRegistry *registry = SPI_REGISTRY (user_data);
-  event_data *evdata;
+  EventData *evdata;
   DBusMessage *reply;
   DBusMessageIter iter, iter_struct, iter_array;
   GList *list;
@@ -999,8 +1046,7 @@ static const char *introspection_footer =
 "</node>";
 
 static DBusMessage *
-impl_Introspect_root (DBusConnection * bus,
-                 DBusMessage * message, void *user_data)
+impl_Introspect_root (DBusMessage * message, SpiRegistry *registry)
 {
   GString *output;
   gchar *final;
@@ -1027,8 +1073,7 @@ impl_Introspect_root (DBusConnection * bus,
 }
 
 static DBusMessage *
-impl_Introspect_registry (DBusConnection * bus,
-                 DBusMessage * message, void *user_data)
+impl_Introspect_registry (DBusMessage * message, SpiRegistry *registry)
 {
   GString *output;
   gchar *final;
@@ -1055,88 +1100,10 @@ impl_Introspect_registry (DBusConnection * bus,
 
 /*---------------------------------------------------------------------------*/
 
-/*
- * Emits an AT-SPI event.
- * AT-SPI events names are split into three parts:
- * class:major:minor
- * This is mapped onto D-Bus events as:
- * D-Bus Interface:Signal Name:Detail argument
- *
- * Marshals a basic type into the 'any_data' attribute of
- * the AT-SPI event.
- */
-static void 
-emit_event (DBusConnection *bus,
-            const char *klass,
-            const char *major,
-            const char *minor,
-            dbus_int32_t detail1,
-            dbus_int32_t detail2,
-            const char *name,
-            const char *path)
-{
-  DBusMessage *sig;
-  DBusMessageIter iter, iter_variant, iter_array;
-  
-  sig = dbus_message_new_signal(SPI_DBUS_PATH_ROOT, klass, major);
-
-  dbus_message_iter_init_append(sig, &iter);
-
-  dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING, &minor);
-  dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &detail1);
-  dbus_message_iter_append_basic(&iter, DBUS_TYPE_INT32, &detail2);
-
-  dbus_message_iter_open_container (&iter, DBUS_TYPE_VARIANT, "(so)",
-                                    &iter_variant);
-    append_reference (&iter_variant, name, path);
-  dbus_message_iter_close_container (&iter, &iter_variant);
-
-  dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{sv}",
-                                    &iter_array);
-  dbus_message_iter_close_container (&iter, &iter_array);
-
-  dbus_connection_send(bus, sig, NULL);
-  dbus_message_unref(sig);
-}
-
-/*---------------------------------------------------------------------------*/
-
-/*
- * Children changed signal converter and forwarder.
- *
- * Klass (Interface) org.a11y.atspi.Event.Object
- * Major is the signal name.
- * Minor is 'add' or 'remove'
- * detail1 is the index.
- * detail2 is 0.
- * any_data is the child reference.
- */
-
-static void
-children_added_listener (DBusConnection * bus,
-                         gint             index,
-                         const gchar    * name,
-                         const gchar    * path)
-{
-  emit_event (bus, SPI_DBUS_INTERFACE_EVENT_OBJECT, "ChildrenChanged", "add", index, 0,
-              name, path);
-}
-
-static void
-children_removed_listener (DBusConnection * bus,
-                           gint             index,
-                           const gchar    * name,
-                           const gchar    * path)
-{
-  emit_event (bus, SPI_DBUS_INTERFACE_EVENT_OBJECT, "ChildrenChanged", "remove", index, 0,
-              name, path);
-}
-
-/*---------------------------------------------------------------------------*/
-
 static DBusHandlerResult
 handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
 {
+  SpiRegistry *registry = SPI_REGISTRY (user_data);
   DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
   const gchar *iface   = dbus_message_get_interface (message);
@@ -1177,13 +1144,13 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
               if (!strcmp (prop_iface, SPI_DBUS_INTERFACE_ACCESSIBLE))
                 {
                   if      (!strcmp (prop_member, "Name"))
-                    impl_get_Name (&iter, user_data);
+                    impl_get_Name (&iter, registry);
                   else if (!strcmp (prop_member, "Description"))
-                    impl_get_Description (&iter, user_data);
+                    impl_get_Description (&iter, registry);
                   else if (!strcmp (prop_member, "Parent"))
-                    impl_get_Parent (&iter, user_data);
+                    impl_get_Parent (&iter, registry);
                   else if (!strcmp (prop_member, "ChildCount"))
-                    impl_get_ChildCount (&iter, user_data);
+                    impl_get_ChildCount (&iter, registry);
                   else
                     {
                       dbus_message_unref (reply); 
@@ -1193,9 +1160,9 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
               else if (!strcmp (prop_iface, SPI_DBUS_INTERFACE_APPLICATION))
                 {
                   if (!strcmp (prop_member, "ToolkitName"))
-                    impl_get_ToolkitName (&iter, user_data);
-                  else if (!strcmp (prop_member, "ToolkitVersion"))
-                    impl_get_ToolkitVersion (&iter, user_data);
+                    impl_get_ToolkitName (&iter, registry);
+                  else if (!strcmp (prop_member, "Version"))
+                    impl_get_ToolkitVersion (&iter, registry);
                   else
                     {
                       dbus_message_unref (reply); 
@@ -1224,27 +1191,27 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "GetChildAtIndex"))
-          reply = impl_GetChildAtIndex (bus, message, user_data);
+          reply = impl_GetChildAtIndex (message, registry);
       else if (!strcmp (member, "GetChildren"))
-          reply = impl_GetChildren (bus, message, user_data);
+          reply = impl_GetChildren (message, registry);
       else if (!strcmp (member, "GetIndexInParent"))
-          reply = impl_GetIndexInParent (bus, message, user_data);
+          reply = impl_GetIndexInParent (message, registry);
       else if (!strcmp (member, "GetRelationSet"))
-          reply = impl_GetRelationSet (bus, message, user_data);
+          reply = impl_GetRelationSet (message, registry);
       else if (!strcmp (member, "GetRole"))
-          reply = impl_GetRole (bus, message, user_data);
+          reply = impl_GetRole (message, registry);
       else if (!strcmp (member, "GetRoleName"))
-          reply = impl_GetRoleName (bus, message, user_data);
+          reply = impl_GetRoleName (message, registry);
       else if (!strcmp (member, "GetLocalizedRoleName"))
-          reply = impl_GetLocalizedRoleName (bus, message, user_data);
+          reply = impl_GetLocalizedRoleName (message, registry);
       else if (!strcmp (member, "GetState"))
-          reply = impl_GetState (bus, message, user_data);
+          reply = impl_GetState (message, registry);
       else if (!strcmp (member, "GetAttributes"))
-          reply = impl_GetAttributes (bus, message, user_data);
+          reply = impl_GetAttributes (message, registry);
       else if (!strcmp (member, "GetApplication"))
-          reply = impl_GetApplication (bus, message, user_data);
+          reply = impl_GetApplication (message, registry);
       else if (!strcmp (member, "GetInterfaces"))
-          reply = impl_GetInterfaces (bus, message, user_data);
+          reply = impl_GetInterfaces (message, registry);
       else
          result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1253,23 +1220,23 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "Contains"))
-          reply = impl_Contains (bus, message, user_data);
+          reply = impl_Contains (message, registry);
       else if (!strcmp (member, "GetAccessibleAtPoint"))
-          reply = impl_GetAccessibleAtPoint (bus, message, user_data);
+          reply = impl_GetAccessibleAtPoint (message, registry);
       else if (!strcmp (member, "GetExtents"))
-          reply = impl_GetExtents (bus, message, user_data);
+          reply = impl_GetExtents (message, registry);
       else if (!strcmp (member, "GetPosition"))
-          reply = impl_GetPosition (bus, message, user_data);
+          reply = impl_GetPosition (message, registry);
       else if (!strcmp (member, "GetSize"))
-          reply = impl_GetSize (bus, message, user_data);
+          reply = impl_GetSize (message, registry);
       else if (!strcmp (member, "GetLayer"))
-          reply = impl_GetLayer (bus, message, user_data);
+          reply = impl_GetLayer (message, registry);
       else if (!strcmp (member, "GetMDIZOrder"))
-          reply = impl_GetMDIZOrder (bus, message, user_data);
+          reply = impl_GetMDIZOrder (message, registry);
       else if (!strcmp (member, "GrabFocus"))
-          reply = impl_GrabFocus (bus, message, user_data);
+          reply = impl_GrabFocus (message, registry);
       else if (!strcmp (member, "GetAlpha"))
-          reply = impl_GetAlpha (bus, message, user_data);
+          reply = impl_GetAlpha (message, registry);
       else
          result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1278,9 +1245,9 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "Embed"))
-          reply = impl_Embed (bus, message, user_data);
+          reply = impl_Embed (message, registry);
       else if (!strcmp (member, "Unembed"))
-          reply = impl_Unembed (bus, message, user_data);
+          reply = impl_Unembed (message, registry);
       else
           result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1289,7 +1256,7 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "Introspect"))
-          reply = impl_Introspect_root (bus, message, user_data);
+          reply = impl_Introspect_root (message, registry);
       else
           result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1317,6 +1284,7 @@ handle_method_root (DBusConnection *bus, DBusMessage *message, void *user_data)
 static DBusHandlerResult
 handle_method_cache (DBusConnection *bus, DBusMessage *message, void *user_data)
 {
+  SpiRegistry *registry = SPI_REGISTRY (user_data);
   DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
   const gchar *iface   = dbus_message_get_interface (message);
@@ -1335,7 +1303,7 @@ handle_method_cache (DBusConnection *bus, DBusMessage *message, void *user_data)
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "GetItems"))
-          reply = impl_GetItems (bus, message, user_data);
+          reply = impl_GetItems (message, registry);
       else
          result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1356,6 +1324,7 @@ handle_method_cache (DBusConnection *bus, DBusMessage *message, void *user_data)
 static DBusHandlerResult
 handle_method_registry (DBusConnection *bus, DBusMessage *message, void *user_data)
 {
+  SpiRegistry *registry = SPI_REGISTRY (user_data);
   DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
   const gchar *iface   = dbus_message_get_interface (message);
@@ -1374,11 +1343,11 @@ handle_method_registry (DBusConnection *bus, DBusMessage *message, void *user_da
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if (!strcmp(member, "RegisterEvent"))
-      reply = impl_register_event (bus, message, user_data);
+      reply = impl_RegisterEvent (message, registry);
       else if (!strcmp(member, "DeregisterEvent"))
-        reply = impl_deregister_event (bus, message, user_data);
+        reply = impl_DeregisterEvent (message, registry);
       else if (!strcmp(member, "GetRegisteredEvents"))
-        reply = impl_get_registered_events (bus, message, user_data);
+        reply = impl_GetRegisteredEvents (message, registry);
       else
           result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1387,7 +1356,7 @@ handle_method_registry (DBusConnection *bus, DBusMessage *message, void *user_da
     {
       result = DBUS_HANDLER_RESULT_HANDLED;
       if      (!strcmp (member, "Introspect"))
-          reply = impl_Introspect_registry (bus, message, user_data);
+          reply = impl_Introspect_registry (message, registry);
       else
           result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
@@ -1441,24 +1410,29 @@ static gchar *app_sig_match_name_owner =
 SpiRegistry *
 spi_registry_new (DBusConnection *bus)
 {
-  SpiRegistry *reg = g_object_new (SPI_REGISTRY_TYPE, NULL);
+  SpiRegistry *registry = g_object_new (SPI_REGISTRY_TYPE, NULL);
+  const char *bus_unique_name;
 
-  reg->bus = bus;
+  bus_unique_name = dbus_bus_get_unique_name (bus);
+  g_assert (bus_unique_name != NULL);
+
+  registry->bus = bus;
+  registry->bus_unique_name = g_strdup (bus_unique_name);
 
   dbus_bus_add_match (bus, app_sig_match_name_owner, NULL);
-  dbus_connection_add_filter (bus, signal_filter, reg, NULL);
+  dbus_connection_add_filter (bus, signal_filter, registry, NULL);
 
-  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_ROOT, &root_vtable, reg);
+  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_ROOT, &root_vtable, registry);
 
-  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_CACHE, &cache_vtable, reg);
+  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_CACHE, &cache_vtable, registry);
 
-  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_REGISTRY, &registry_vtable, reg);
+  dbus_connection_register_object_path (bus, SPI_DBUS_PATH_REGISTRY, &registry_vtable, registry);
 
   emit_Available (bus);
 
-  reg->events = NULL;
+  registry->events = NULL;
 
-  return reg;
+  return registry;
 }
 
 /*END------------------------------------------------------------------------*/
